#!/bin/bash


if [ "$1" = "-h" ] || [ "$1" = "-help" ] || [ "$1" = "--help" ];then
  cat<<EOF

Build Puppy Linux PET packages from Crux Linux Pkgfiles

USAGE

  crux_build.sh <pkgname>    # build from /usr/local/src/<pkgname>/Pkgfile
  crux_build.sh --help       # show this help

EXAMPLE

  crux_build.sh bftp

REQUIREMENTS

  1. put the Pkgfile in /usr/local/src/<pkgname>/Pkgile
  2. put other files referenced by the Pkgfile in there too
  3. optionally create /usr/local/src/<pkgname>/README
EOF
  exit 0
fi


# set the vars needed by Pkfiles

SRC=""     # the dir containing the source code, Makefile, etc
PKG=""     # the name of the package dir
name=""    # the name of the package (not inc version)
version="" # the version of the package
release="" # the build number
source=""  # the sources (a bash array of URLs and local files)

# error checking
[ ! -d /usr/local/src ] && { echo "ERROR: /usr/local/src doesn't exist!" && exit 1; }
[ ! -d /usr/local/src/"$1" ] && { echo "ERROR: /usr/local/src/$1 doesn't exist!" && exit 1; }
[ ! -f /usr/local/src/"$1"/Pkgfile ] && { echo "No Pkgfile found in /usr/local/src/$1/" && exit 1; }
grep -qE '^name=' /usr/local/src/$1/Pkgfile || { echo "/usr/local/src/$1/Pkgfile is not a valid build script" && exit 1; }

# define a cleanup function for to be used later
cleanup() {
  [ -d "${name}" ] && rm -rf "${name}"
  [ -d "${name}-${version}" ] && rm -rf "${name}-${version}"
  [ -d "$PKG" ] && rm -rf "$PKG"
  [ -f nohup.out ] && rm nohup.out
}

# get info about this puppy version
source /etc/DISTRO_SPECS

# go into the src/<pkgname> dir
cd "/usr/local/src/$1"
# clear old files
[ -d "$PKG" ] && rm -rf "$PKG"
[ -f ${PKG}.pet ] && rm ${PKG}.pet
# get info from Pkgfile build the package dir name
source Pkgfile
PKG="${name}-${version}-${DISTRO_FILE_PREFIX}-${DISTRO_VERSION}-${release:-1}-${DISTRO_TARGETARCH}"

for f in ${source[@]}
do
  downloaded_file="$(basename "$f")"
  # if file is a URL, download it
  case $f in
    *'://'*)
      #[ -f "$downloaded_file" ] && rm "$downloaded_file"
      echo "Downloading $f"
      wget "$f"  || { echo "Error, cant download '$f'" && exit 1; }
      ;;
  esac
  if [ ! -f "$downloaded_file" ];then
    downloaded_file="$(find -type f -name "${name}-$version*")"
  fi
  if [ ! -f "$downloaded_file" ];then
    downloaded_file="$(find -type f -name "${name}-$commit*")"
  fi
  if [ ! -f "$downloaded_file" ];then
    downloaded_file="$(find -type f -name "$commit*")"
  fi
  if [ ! -f "$downloaded_file" ];then
    downloaded_file="$(find -type f -name "$f*")"
  fi
  # if file was downloaded, extract it
  if [ -f "$downloaded_file" ];then
    case $(file --mime-type -b "$downloaded_file") in
      *tar*|*xz*|*bzip*|*zip)
        echo extract "$downloaded_file"
        extract "$downloaded_file" || { echo "Could not extract source!" && exit 1; }
        ;;
    esac
  else
    echo "File $downloaded_file not downloaded"
    exit 1
  fi
done

# set the dir we're in for later
maindir="$PWD"

# $SRC is used by Pkgfiles to refer to the unzipped folder,
# containing the course code.. so lets find it,,
SRC="$PWD/$(find -type d -name "${name}*" | head -1)"

# make the package dir
[ -d "$PKG" ] && rm -rf "$PKG"
mkdir "$PKG" || { echo "Error: Couldn't make dir '$PKG'" && exit 1; }

echo -e "\nBuilding into $PKG..\n"

# set this to be sure, we may 'cd' into another dir in the build function
PKG="${maindir}/$PKG"

# function sometimes used by Pkgfile
pkginfo() {
  [ "$1" = "-i " ] && sort -u $HOME/.packages/builtin_files*/ && return 0
}

build 1>/dev/null || {
  sleep 5
  make clean 1>/dev/null
  cd "$maindir"
  echo -e "\n\nBuild failed!"
  cleanup
  exit 1
}

# if we did cd, go back to main folder (containing Pkgfile)
[ "$PWD" != "$maindir" ] && cd "$maindir"

# check the source compiled
[ ! -d "$PKG" ] && { echo "Error: $PKG is not a dir" && exit 1; }

# get list of files that'll be in the new package
files="$(find $(basename "$PKG") -iname "*")"

# and compiled into the right dir
if [ "$files" = "" ];then
  echo "Error: $PKG dir is empty!"
  cleanup
  exit 1
fi

# lets build the PET package
dir2pet $(basename "$PKG")

if [ -f "${PKG}.pet" ];then
  # final msg
  echo -e "\nSuccess!\n"
  md5sum $(basename "${PKG}.pet")
  echo -e "\n$files\n"
  # show a readme of the program
  [ -f README ] && less README
fi

echo -e "\nCleaning up..."
cleanup

exit 0
